#!/bin/sh
# SPDX-License-Identifier: MIT
# Copyright (C) 2019 Beckhoff Automation GmbH & Co. KG
# Author: Julian Heuking <j.heuking@beckhoff.com>

set -u

shlib="${shlib-"/usr/local/bin/shlib"}"
. "${shlib}/dialog_std.sh"

DIALOG="${DIALOG-"dialog"}"

get_installer_disk_regex() {
	installer_part="$(glabel status | grep -E "(^[[:space:]]+|^)ufs/INSTALLER[[:space:]]+" | awk '{print $NF}')"
	installer_disk="$(geom part status | grep -E "(^[[:space:]]+|^)${installer_part}" | awk '{print $NF}')"

	if ! [ -z "${installer_disk}" ]; then
		printf "^${installer_disk}[[:space:]]+"
		return 0
	fi

	return 1
}

# shows available disks with their size in a dialog and returns the choosen disk
# example:
#	local _choosen_disk=
#	dialog_disk_selection "_choosen_disk" "choose your disk\n"
#	[ $? -eq 0 ] && printf "${_choosen_disk}\n" || printf "dialog aborted\n"
dialog_disk_selection() {
	local _ref_dialog_result=${1}
	local _dlg_msg=${2-"Please choose a disk\n"}
	local _exclusions=${3-"^$"}

	# build list in this pattern: disk (size),description
	# f.e. ada0 (15G),ADATA_FGN4321HG
	local _available_disks=$(geom disk list | awk '/Name:/ {printf $NF " "} /Mediasize:/ {printf $NF ","} /descr:/ {for (i=2; i<NF; i++) printf $i "_"; print $NF}' | grep -v -E "${_exclusions}")

	if [ -z "${_available_disks}" ]; then
		dialog_msgbox "no disks available"
		return 1
	fi

	# dialog only takes lines pairs of two, so filter out any lines with field count not matching
	# cut the description line if is is getting too long
	_dialog_parameter=$(printf '%s\n' "${_available_disks}" \
		| awk '{if (NF==2 && length($2)>35) {printf $1" "substr($2, 1, 35)".. "} else if(NF==2) printf $0" "}')

	while :; do
		local _disk=
		_disk=$(${DIALOG} \
			--title "Disk Selection" ${DIALOG_BACKTITLE+"--backtitle"} ${DIALOG_BACKTITLE+"${DIALOG_BACKTITLE-}"} \
			--menu "${_dlg_msg}" \
			14 48 5 \
			${_dialog_parameter} \
			3>&1 1>&2 2>&3)

		case $? in
			0)
				if [ -z "${_disk}" ]; then
					dialog_msgbox "No disk selected"
					continue
				fi
				eval ${_ref_dialog_result}="\"${_disk}\""
				return 0
				;;
			1)
				return 1
				;;
			255)
				return 1
				;;
			*)
				dialog_msgbox "dialog: internal error"
				return 2
				;;
		esac
	done
}

# Show a (red) colored warning: ALL DATA WILL BE LOST
# the parameter $_pre_msg is optional and will be prepended before the warning, but is black colored
# the dialog defaults to "no"-button
# usage:
#	dialog_data_loss_warning "ada0" "Restoring backup xyz..."
dialog_data_loss_warning() {
	local _disk="${1}"
	local _pre_msg="${2-""}"

	${DIALOG} --title "Warning" \
		--colors ${DIALOG_BACKTITLE+"--backtitle"} ${DIALOG_BACKTITLE+"${DIALOG_BACKTITLE-}"} \
		--defaultno \
		--yesno "${_pre_msg}\Zb\Z1WARNING! ALL DATA ON ${_disk} WILL BE LOST\Zn\ncontinue?" 0 0

	case $? in
		0) # yes
			return 0
			;;
		1 | 255) # no / esc
			return 1
			;;
		*)
			dialog_msgbox "dialog: internal error"
			return 2
			;;
	esac
}
