#!/bin/sh
# SPDX-License-Identifier: MIT
# Copyright (C) 2019 Beckhoff Automation GmbH & Co. KG
# Author: Julian Heuking <j.heuking@beckhoff.com>

set -u

shlib="${shlib-"/usr/local/bin/shlib"}"
. "${shlib}/dialog_std.sh"

DIALOG="${DIALOG-dialog}"

# Show password dialog with option to display the password in the input box
# If the user hits 'ok' the variable specified with --outvar will hold the
# password
#
dialog_password() {
	# if _prompt is already set from a previous call, we should
	# not override it
	local _prompt="${_prompt-"Please enter a password"}"

	# default to not showing the password
	local _show_password="0"
	while [ $# -gt 0 ]; do
		case $1 in
			--prompt)
				shift
				_prompt="${1}"
				;;
			--show-password)
				_show_password="1"
				;;
			--outvar)
				shift
				local _ref_output="${1}"
				;;
			*)
				printf "unknown argument: $1\n" >&2
				return 1
				;;

		esac
		shift
	done

	if [ -z "${_ref_output-""}" ]; then
		printf "dialog_password: invalid arguments!" >&2
		return 1
	fi

	# switch between passwordbox and inputbox to display stars
	# or the actual password in the input field
	if [ "${_show_password}" = "0" ]; then
		local _extra_label="show password"
		local _box="--insecure --passwordbox"
	else
		local _extra_label="hide password"
		local _box="--inputbox"
	fi

	# we use _dialog_out as INIT value here, so that the
	# password is still entered in the next dialog when the
	# hide/show password button is pressed
	local _dialog_out
	_dialog_out=$(${DIALOG} \
		--title "${DIALOG_TITLE-"Password"}" \
		--backtitle "${DIALOG_BACKTITLE-"Password"}" \
		--ok-label "${DIALOG_MSG_OK-"ok"}" \
		--cancel-label "${DIALOG_MSG_CANCEL-"cancel"}" \
		--extra-button \
		--extra-label "${_extra_label}" \
		${_box} "${_prompt}" \
		0 0 ${_dialog_out-""} \
		3>&1 1>&2 2>&3)
	case $? in
		0) #ok
			# write value to user-specified output variable
			setvar "${_ref_output}" "${_dialog_out}"
			;;
		1 | 255) # cancel
			return 1
			;;
		3) # hide/show password
			# alike static c/c++ variables, local variables in
			# dash persist in scope of the function
			# so there is no need to pass all arguments again here
			dialog_password $(test "${_show_password}" = "0" && printf '%s' "--show-password")
			;;
		*)
			dialog_msgbox "internal dialog error\n"
			return 4
			;;
	esac
}

# Prompt the user to enter a password (twice). If the user does not cancel or
# press ESC, the variable specified by --outvar will hold the password
#
dialog_password_routine() {
	local _prompt1="Please enter a password"
	local _prompt2="Re-enter password"
	local _ref_output=
	while [ $# -gt 0 ]; do
		case $1 in
			--outvar)
				shift
				_ref_output="${1}"
				;;
			--prompt1)
				shift
				_prompt1="${1}"
				;;
			--prompt2)
				shift
				_prompt2="${1}"
				;;
			--title)
				shift
				DIALOG_TITLE="${1}"
				;;
			--backtitle)
				shift
				DIALOG_BACKTITLE="${1}"
				;;
			*)
				dialog_msgbox "Invalid argument ${1}\n"
				return 1
				;;
		esac
		shift
	done

	if [ -z "${_ref_output}" ]; then
		dialog_msgbox "output variable is undefined\n"
		return 1
	fi

	#
	# Loop until the user provides taint-free/valid input
	#
	local _password1=
	local _password2=
	while :; do
		if ! dialog_password --prompt "${_prompt1}" --outvar "_password1"; then
			return 1
		fi

		if ! dialog_password --prompt "${_prompt2}" --outvar "_password2"; then
			return 1
		fi

		# Check for NULL entry
		if ! [ "$_password1" -o "$_password2" ]; then
			dialog_msgbox "Passwords cannot be empty!"
			continue
		fi

		# Check for password mismatch
		if [ "$_password1" != "$_password2" ]; then
			dialog_msgbox "Passwords do not match!"
			continue
		fi

		setvar "${_ref_output}" "$_password1"
		break
	done

	return 0
}
