#!/bin/sh
# SPDX-License-Identifier: MIT
# Copyright (C) 2019 - 2020 Beckhoff Automation GmbH & Co. KG
# Author: Julian Heuking <j.heuking@beckhoff.com>
#
# This is a collection of generic log functions to be reused by other scripts
# source this script and log with:
# logerr "FATAL ERROR\n"
# or
# loginfo "Everything is working as expected\n"
# you can pipe into the log function, too:
# echo "Another error happend" | logerr

# Default log level, overwrite in your own script
# 1 - error -> something bad happend, usually we stop further processing
# 2 - warning -> something went wrong, but it's not critical and we might proceed
# 3 - info -> we want to inform about progress, usually these messages can be ignored
# 4 - verbose -> excessive logging, we want to see this messages only during debugging
BHF_LOG_LEVEL=${BHF_LOG_LEVEL-2}

# Default log file, overwrite in your own script
BHF_LOG_FILE=${BHF_LOG_FILE-"/dev/null"}

set -u

logmsg() {
	printf "$@" | tee -a "${BHF_LOG_FILE}" 1>&2
}

logstdin() {
	tee -a "${BHF_LOG_FILE}" 1>&2
}

log() {
	exec 3>&2
	if [ ${1} -gt ${BHF_LOG_LEVEL} ]; then
		exec 2> /dev/null
	fi
	shift

	if [ $# -eq 0 ]; then
		logstdin
	else
		logmsg "$@"
	fi

	exec 2>&3 3>&-
}

loginfo() {
	log 3 "$@"
}

logwarn() {
	log 2 "$@"
}

logerr() {
	log 1 "$@"
}

selftest_run() {
	BHF_LOG_FILE="$0-selftest-${BHF_LOG_LEVEL}.log"
	rm -f "${BHF_LOG_FILE}"

	echo "hello world" | logerr
	logerr "%s" "1 2" 3 4 '|ü'
	logerr "*$\n"
	echo "hello world" | loginfo
	loginfo "%s" "1 2" 3 4 '|ü'
	loginfo "*$\n"
}

log_selftest() {
	local _expected_string="hello world\n1 234|ü*$\n"
	local _old_level="${BHF_LOG_LEVEL}"
	local _old_file="${BHF_LOG_FILE}"
	local _tmpfile=$(mktemp)
	local _result=0

	BHF_LOG_LEVEL=2
	selftest_run 2> ${_tmpfile}
	printf "${_expected_string}" | diff ${_tmpfile} - || _result=$?
	printf "${_expected_string}${_expected_string}" | diff ${BHF_LOG_FILE} - || _result=$?

	BHF_LOG_LEVEL=4
	selftest_run 2> ${_tmpfile}
	printf "${_expected_string}${_expected_string}" | diff ${_tmpfile} - || _result=$?
	printf "${_expected_string}${_expected_string}" | diff ${BHF_LOG_FILE} - || _result=$?

	BHF_LOG_LEVEL="${_old_level}"
	BHF_LOG_FILE="${_old_file}"
	rm ${_tmpfile}
	return ${_result}
}
