#!/bin/sh
# SPDX-License-Identifier: MIT
# Copyright (C) 2019 Beckhoff Automation GmbH & Co. KG
# Author: Julian Heuking <j.heuking@beckhoff.com>
#
# This is a collection of functions to simplify zfs related

. /usr/local/bin/shlib/log.sh

set -u

# run 'zfs list' for different types, show 'names' only
# usage: list=$(zfs_list <type>)
# f.e.:  list=$(zfs_list "filesystem")
# f.e.:  list=$(zfs_list "snapshot")
zfs_list() {
	if ! zfs list -H -o name -t "${1}"; then
		logerr "failed to lookup zfs ${1}\n"
		return 1
	fi
}

# lists imported pools with 'zpool list'
# usage: list=$(zpool_list)
zpool_list() {
	if ! zpool list -H -o name; then
		logerr "failed to lookup zpool names\n"
		return 1
	fi
}

# get zpool property
# usage: property=$(zpool_get <property> <zpool>)
# f.e.:  property=$(zpool_get "bootfs" "zroot")
zpool_get() {
	local _property="${1}"
	local _pool="${2}"

	if ! zpool get -H -o value "${_property}" "${_pool}"; then
		logerr "failed to lookup '${_property}' from '${_pool}'\n"
		return 1
	fi
}

# get zfs property
# usage: property=$(zfs_get <property> [<filesystem/snapshot/volume>])
# f.e.:  property=$(zfs_get "mountpoint" "zroot/ROOT/default")
# f.e.:  property=$(zfs_get "mountpoint")
zfs_get() {
	local _property="${1}"
	local _type="${2-""}"

	if ! zfs get -H -o value "${_property}" "${_type}"; then
		logerr "failed to lookup '${_property}' from '${_type-"all"}'\n"
		return 1
	fi
}

# set a zpool property
# usage: zpool_set <property> <value> <zpool>
# f.e.:  zpool_set "bootfs" "zroot/ROOT/default" "zroot"
zpool_set() {
	local _property="${1}"
	local _value="${2}"
	local _pool="${3}"

	if ! zpool set "${_property}"="${_value}" "${_pool}"; then
		logerr "failed to set '${_property}' for '${_pool}'\n"
		return 1
	fi
}

# set zpool "bootfs" property
# usage: zpool_set_bootfs <dataset_to_boot> [<zpool>]
# f.e.:  zpool_set_bootfs "zroot/ROOT/default"
#        zpool_set_bootfs "zroot/ROOT/default" zother
zpool_set_bootfs() {
	# read pool name from bootfs parameter if not provided
	local _pool="${2-${1%%/*}}"
	local _bootfs="${1}"

	zpool_set "bootfs" "${_bootfs}" "${_pool}"
}

# get dataset which is mounted on /
# usage: dataset=$(zpool_get_booted_dataset)
zpool_get_booted_dataset() {
	local _bootfs=$(df / | tail -1 | awk '{print $1}')
	if [ -z "${_bootfs}" ]; then
		logerr "could not lookup bootfs\n"
		return 1
	fi

	# check if we actually found a dataset
	if ! zfs_list "filesystem" | grep "${_bootfs}" > /dev/null; then
		logerr "${_bootfs} is not a zfs dataset\n"
		return 2
	fi

	printf "${_bootfs}\n"
}
